@@ -11,6 +11,8 @@ from __future__ import absolute_import, division, print_function
 
 import asyncio
 from typing import List, Iterable
+assert List is not None
+assert Iterable is not None
 import os
 
 from langchain_core.documents import Document
@@ -227,25 +229,25 @@ Be conservative - only report metrics you're confident are present.
 def detect_program_from_text(text: str) -> str:
     """
     Detects the Phenix program name from log text.
-    
+
     Checks multiple patterns:
     1. Client-added header "COMMAND THAT WAS RUN: phenix.xxx ..."
     2. Standard Phenix log header "Starting phenix.xxx on ..."
     3. Program-specific content patterns
-    
+
     Args:
         text: Log file content
-        
+
     Returns:
         Program name (e.g., "phenix.refine") or empty string
     """
     if not text:
         return ""
-    
+
     # Check first 5000 chars for efficiency
     text_start = text[:5000]
     text_start_lower = text_start.lower()
-    
+
     # Pattern 1: Client-added header "COMMAND THAT WAS RUN: phenix.xxx ..."
     for line in text_start.splitlines():
         if "COMMAND THAT WAS RUN:" in line:
@@ -254,7 +256,7 @@ def detect_program_from_text(text: str) -> str:
                 if p.startswith("phenix."):
                     debug_print(f"Detected program from COMMAND header: {p}")
                     return p
-    
+
     # Pattern 2: Standard Phenix log header "Starting phenix.xxx on ..."
     for line in text_start.splitlines():
         line_stripped = line.strip()
@@ -264,7 +266,7 @@ def detect_program_from_text(text: str) -> str:
                 if p.startswith("phenix."):
                     debug_print(f"Detected program from 'Starting' header: {p}")
                     return p
-    
+
     # Pattern 3: Program-specific content patterns
     # Order matters - check more specific patterns first
     program_patterns = [
@@ -281,13 +283,13 @@ def detect_program_from_text(text: str) -> str:
         ("phenix.dock_in_map", ["phenix.dock_in_map", "dock_in_map"]),
         ("phenix.autosol", ["phenix.autosol", "autosol", "sad phasing", "experimental phasing"]),
     ]
-    
+
     for program_name, patterns in program_patterns:
         for pattern in patterns:
             if pattern in text_start_lower:
                 debug_print(f"Detected program from content pattern '{pattern}': {program_name}")
                 return program_name
-    
+
     debug_print("Could not detect program from text")
     return ""
 
@@ -296,29 +298,23 @@ def detect_program_from_text(text: str) -> str:
 # =============================================================================
 
 def get_log_map_prompt(program_name: str = "") -> PromptTemplate:
-    """
-    Returns the prompt for summarizing a single chunk of a log file.
-    
-    Args:
-        program_name: Optional program name for specific guidance
-        
-    Returns:
-        PromptTemplate configured for the program type
-    """
+    # 1. Get the guidance string
     program_guidance = get_program_specific_guidance(program_name)
-    
-    template = f"""You are a crystallography data extraction expert analyzing a Phenix log file.
 
-{program_guidance}
+    # 2. Use a standard string (no 'f') and use single braces for all variables
+    #    We will "hard code" the guidance into the template using .partial()
+    template = """You are a crystallography data extraction expert analyzing a Phenix log file.
+
+{program_guidance_text}
 
 **CRITICAL INSTRUCTIONS:**
 1. Analyze the ENTIRE log text below - not just the beginning or end
-2. Extract ONLY information that is explicitly present in the log
-3. Do NOT invent or assume information not shown
-4. For output files, only list files explicitly shown as "written to" or "Writing:" or "Output:"
+2. I will provide a log file summary enclosed in <log_data> tags.
+3. Extract ONLY information that is explicitly present in the log
+4. Do NOT invent or assume information not shown
+5. For output files, only list files explicitly shown as "written to" or "Writing:" or "Output:"
 
 **EXTRACT THE FOLLOWING (if present):**
-
 1. PROGRAM NAME: The specific phenix program (e.g., phenix.xtriage, phenix.phaser)
 2. INPUT FILES: File paths for .mtz, .pdb, .fa, .seq files
 3. OUTPUT FILES: ONLY files explicitly written (look for "Writing:", "written to", "Output:")
@@ -330,12 +326,19 @@ def get_log_map_prompt(program_name: str = "") -> PromptTemplate:
 
 ---
 LOG CHUNK TO ANALYZE:
-"{{text}}"
+<log_data>
+{text}
+</log_data>
 ---
 
 Extracted information:
 """
-    return PromptTemplate.from_template(template)
+    # 3. Create template and inject guidance immediately
+    prompt = PromptTemplate.from_template(template)
+
+    # This "bakes in" the guidance safely, even if it has special characters
+    return prompt.partial(program_guidance_text=program_guidance)
+
 
 def get_log_combine_prompt(program_name: str = "") -> PromptTemplate:
     """
@@ -634,7 +637,7 @@ async def summarize_log_text(
 
         try:
             map_results = await asyncio.wait_for(asyncio.gather(*tasks), timeout=timeout)
-            
+
             for j, result in enumerate(map_results):
                 summary_text = result.content
                 all_intermediate_summaries.append(summary_text)
@@ -678,7 +681,7 @@ async def summarize_log_text(
     # NEW: Use program-specific combine prompt
     combine_prompt = get_log_combine_prompt(program_name)
 
-    
+
     # Debug: show the combine prompt
     debug_print("COMBINE PROMPT TEMPLATE:")
     debug_print(combine_prompt.template[:500] + "...")
@@ -696,4 +699,3 @@ async def summarize_log_text(
         log_summary=final_output,
         error=None
     )
-
